/*! \file date.h
 *  \brief Date class declaration.
 */

#ifndef DATE_H
#define DATE_H

#include <ctime> 

/**
 * \class Date 
 * \brief Object representing a date as a three integers and a string.
 */
class Date {
    public:
        /*! \brief Date default constructor */
        Date();

        /*! \brief Date copy constructor */
        Date(const Date& date);

        /*! 
         * \brief Date constructor 
         * \param day An integer between 1 and 31 representing the day.
         * \param month An integer between 1 and 12 representing the month.
         * \param year An integer greater than 1970 that represents the year.
         * \param current_day An array of chars representing the weekday.
         * \pre \f$ \mathrm{day} \in [\![ 1, 31 ]\!] \f$
         * \pre \f$ \mathrm{month} \in [\![ 1, 12 ]\!] \f$
         * \pre \f$ \mathrm{year} \in [\![ 1970, \mathrm{INT\_MAX} ]\!] \f$
         * \note If \ref current_day is <TT>NULL</TT>, it will be automatically determined based on the input date by using
         * <a href="https://en.cppreference.com/w/cpp/chrono/c/mktime"><TT>std::mktime</TT></a>
         * and \ref weekdays.
         */
        Date(int day, int month, int year, 
                const char* current_day=nullptr);

        /*! 
         * \brief Date constructor 
         * \param date An opaque type representing ellapsed time since <TT>UNIX Epoch</TT>.
         * \note All \ref Date attributes are automatically determined by using 
         * <a href="https://en.cppreference.com/w/cpp/chrono/c/localtime"><TT>std::locatime</TT></a>
         * and \ref weekdays.
         */
        Date(std::time_t date);

        /*! \brief Date destructor */
        ~Date();
            
        /*! \brief Print the date to standard output. */
        void print_date() const;

        /*! 
         * \brief Print a happy birthday message.
         * \param n Name
         * \param b Birthday date.
         * This method wishes a happy birthday to input name of the birthday date matches today.
         */
        void happy_birthday(const char *n, const Date &b) const;
    
        /*! \brief Returns \ref m_day. */
        int day() const;
        /*! \brief Returns \ref m_month. */
        int month() const;
        /*! \brief Returns \ref m_year. */
        int year() const;
        /*! \brief Returns \ref m_day_str as a const char*. */
        const char* day_str() const;
    private:
        /*! \brief Day corresponding to the date, \f$ \mathrm{m\_day} \in [\![ 1, 31 ]\!] \f$. */
        int m_day;
        /*! \brief Month corresponding to the date, \f$ \mathrm{m\_month} \in [\![ 1, 12 ]\!] \f$. */
        int m_month;
        /*! \brief Year corresponding to the date, \f$ \mathrm{m\_year} \in [\![ 1970, \mathrm{INT\_MAX} ]\!] \f$. */
        int m_year;
        /*! \brief String corresponding to the day of the week. */
        char* m_day_str; 
};

#endif /* end of include guard: DATE_H */
