/*! \file trip.h
 *  \brief Trip class declaration.
 */

#ifndef TRIP_H
#define TRIP_H

#include "date.h"

/**
 * \class Trip
 * \brief Object representing a trip as a start and end \ref Date and a floating point price.
 */
class Trip {
    public:
        /*! 
         * \brief Trip constructor 
         * \param start_date The start \ref Date of the trip.
         * \param end_date The end \ref Date of the trip.
         * \param price The price of the trip.
         * \pre \f$ \mathrm{price} > 0 \f$
         * \pre \f$ \mathrm{start\_date} < \mathrm{end\_date} \f$
         * \note Start date should happen before end date and the price should be stricly positive.
         * \bug Currently we only support trips that begin and end the same year.
         */
        Trip(Date start_date, Date end_date, float price);

        /*! 
         * \brief Trip constructor 
         * \param sday An integer between 1 and 31 representing the day of the start date.
         * \param smonth An integer between 1 and 12 representing the month of the start date.
         * \param syear An integer greater than 1970 that represents the year of the start date.
         * \param eday An integer between 1 and 31 representing the day of the end date.
         * \param emonth An integer between 1 and 12 representing the month of the end date.
         * \param eyear An integer greater than 1970 that represents the year of the end date.
         * \param price The price of the trip.
         * \pre \f$ \mathrm{sday} \in [\![ 1, 31 ]\!] \f$
         * \pre \f$ \mathrm{smonth} \in [\![ 1, 12 ]\!] \f$
         * \pre \f$ \mathrm{syear} \in [\![ 1970, \mathrm{INT\_MAX} ]\!] \f$
         * \pre \f$ \mathrm{eday} \in [\![ 1, 31 ]\!] \f$
         * \pre \f$ \mathrm{emonth} \in [\![ 1, 12 ]\!] \f$
         * \pre \f$ \mathrm{eyear} \in [\![ 1970, \mathrm{INT\_MAX} ]\!] \f$
         * \pre \f$ \mathrm{price} > 0 \f$
         * \note Start date should happen before end date and the price should be stricly positive.
         * \bug Currently we only support trips that begin and end the same year.
         */
        Trip(int sday, int smonth, int syear,
             int eday, int emonth, int eyear,
             float price);

        /*! \brief Print the trip to standard output. */
        void print_trip() const;

        /*! \brief Returns the day averaged price of the trip. */
        float price_per_day() const;

    private:
        /*! \brief Start date of the trip. */
        Date m_start_date;
        /*! \brief End date of the trip. */
        Date m_end_date;
        /*! \brief Price of the trip. */
        float m_price;
};


#endif /* end of include guard: TRIP_H */
