/*! \file date.cpp
 *  \brief Date class implementation.
 */

#include <iostream> /* std::cout, std::endl    */
#include <iomanip>  /* std::setfill, std::setw */
#include <cstring>  /* std::strcpy, std::strlen */

#include "date.h"   /* Date class definition + ctime */

/*! \brief Static array containg the name of the weekdays */
const char* weekdays[7] = {
    "Sunday",
    "Monday", "Tuesday", "Wednesday",
    "Thursday", "Friday", "Saturday"
};

/* default constructor */
Date::Date() : Date(std::time(nullptr)) {
    /* From C++11 you can delegate constructors         *
     * (ie. call another constructor of the same class) */
}


/* copy constructor */
Date::Date(const Date& date):
    m_day(date.day()), m_month(date.month()), m_year(date.year()), // <= requires const ascessors !
    m_day_str(nullptr) {

    /* We need to dynamically allocate our own array of char */
    m_day_str = new char[std::strlen(date.day_str()) + 1]; /* do not forget termination character '\0' ! */
    std::strcpy(m_day_str, date.day_str());
}


Date::Date(int day, int month, int year, 
            const char* current_day):
    m_day(day), m_month(month), m_year(year), 
    m_day_str(nullptr) {

    if ((day<1) || (day>31) || (month<1) || (month>12) || (year<1900)) {
        m_day   = 1;
        m_month = 1;
        m_year  = 1900;
        std::cerr << "Got bad aguments: "
                  << "(day="    << day 
                  << ", month=" << month
                  << ", year="  << year
                  << "), creating default date with Unix Epoch."
                  << std::endl;
    }
    
    if (current_day == nullptr) {
        /* We need to determine current day by using std::mktime *
         * See http://www.cplusplus.com/reference/ctime/mktime/  */
        std::time_t rawtime = std::time(nullptr);
        struct tm *timeinfo = localtime(&rawtime);
        timeinfo->tm_year = m_year  - 1900;
        timeinfo->tm_mon  = m_month - 1;
        timeinfo->tm_mday = m_day;
        std::mktime(timeinfo); /* this sets timeinfo->tm_wday */
        current_day = weekdays[timeinfo->tm_wday];
    }
    m_day_str = new char[std::strlen(current_day) + 1];
    std::strcpy(m_day_str, current_day);
}


Date::Date(std::time_t date):
    m_day(0), m_month(1), m_year(1900),
    m_day_str(nullptr) {
    
    struct tm *lt = std::localtime(&date);
    m_day   += lt->tm_mday;
    m_month += lt->tm_mon;
    m_year  += lt->tm_year;
    
    const char* current_day = weekdays[lt->tm_wday];
    m_day_str = new char[std::strlen(current_day) + 1];
    std::strcpy(m_day_str, current_day);
}


Date::~Date() {
    delete[] m_day_str;
}


void Date::print_date() const {
    std::cout << m_day_str << ", " 
              << std::setfill('0') << std::setw(2) << m_day << '/'
              << std::setfill('0') << std::setw(2) << m_month << '/'
              << m_year;
}


void Date::happy_birthday(const char* n, const Date &b) const {
    if ((m_day == b.m_day) && (m_month == b.m_month)) {
        std::cout << "Happy " << m_year - b.m_year << "th birthday " << n << " !";
    }
    else {
        std::cout << "Sorry, no gifts for you " << n << '.';
    }
}


/* Const correct accessors */
int Date::day()   const { return m_day;   }
int Date::month() const { return m_month; }
int Date::year()  const { return m_year;  }
const char* Date::day_str() const { return m_day_str; }

