
#include <cassert>  /* assert */
#include <cstring>  /* std::memcpy */
#include <cmath>    /* std::isnan */
#include <iostream>
#include "piecewise.h"
 
PiecewisePoly1::PiecewisePoly1(
               const float* Xi,
               const float* Yi,
               unsigned int npoints):
    m_npoints(npoints), m_Xi(nullptr), m_Yi(nullptr), m_polys(nullptr) {

    /* Check that Xi is sorted and do not contain any NaN value */
    assert(npoints>=2);
    for (unsigned int i=0; i<npoints; i++) {
        assert(Xi[i] == Xi[i]);                      /* check for NaN */
        assert(!std::isnan(Yi[i]));                  /* check for NaN */
        assert((i==npoints-1) || (Xi[i] < Xi[i+1])); /* check for strict ordering */
    }

    /* Allocate and copy Xi and Yi values */
    m_Xi = new float[npoints];
    m_Yi = new float[npoints];
    std::memcpy(m_Xi, Xi, npoints*sizeof(float));
    std::memcpy(m_Yi, Yi, npoints*sizeof(float));

    /* Create the polynomials */
    m_polys = new Poly1[npoints-1];
    for (unsigned int i=0; i<npoints-1; i++) {
        /* Pi(x) = a0 + a1*x                      *
         * Pi(Xi)=Yi       =>   a0 + a1*Xl = Yl   *
         * Pi(Xi+1)=Yi+1   =>   a0 + a1*Xr = Yr   */
        const float Xl=Xi[i], Xr=Xi[i+1];
        const float Yl=Yi[i], Yr=Yi[i+1];
        const float a0 = (Yl*Xr-Yr*Xl)/(Xr-Xl);
        const float a1 = (Yr-Yl)/(Xr-Xl);
        m_polys[i] = Poly1(a0, a1);
    }
}

PiecewisePoly1::~PiecewisePoly1() {
    delete[] m_Xi;
    delete[] m_Yi;
    delete[] m_polys;
}

float PiecewisePoly1::xmin() const {
    return m_Xi[0];
}

float PiecewisePoly1::xmax() const {
    return m_Xi[m_npoints-1];
}

float PiecewisePoly1::val(float x) const {
    if (x < xmin())
        return m_Yi[0];
    for (unsigned int i=0; i<m_npoints-1; i++) {
        if (x < m_Xi[i+1])
            return m_polys[i].val(x);
    }
    return m_Yi[m_npoints-1];
}

void PiecewisePoly1::dump(std::ostream& os, unsigned int N) const {
    assert(N>=2);    
    const float dx = (xmax()-xmin())/(N-1);
    float x = xmin();
    for (unsigned int i=0; i<N; i++) {
        os << x << '\t' << val(x) << std::endl;
        x += dx;
    }
}

